declare type ActionHandler<T extends unknown> = (params: T) => void;
interface Event {
    type: string;
    action: ActionHandler<any>;
}
declare class BaseApplication<E extends Event> {
    canvas: HTMLCanvasElement;
    _events: Map<string, Map<string, ActionHandler<unknown>>>;
    _disposers: Set<() => void>;
    constructor(canvas: HTMLCanvasElement);
    on(event: E): () => void;
    dispose(): void;
}
declare class CtxApplication<E extends Event> extends BaseApplication<E> {
    ctx: CanvasRenderingContext2D;
    constructor(canvas: HTMLCanvasElement);
}

declare type Constructor<T = {}> = new (...args: any[]) => T;

interface ResizeHandler<T = ResizeApplication> extends ActionHandler<{
    app: T;
    width: number;
    height: number;
}> {
}
interface ResizeEvent<T = ResizeApplication> extends Event {
    type: 'resize';
    action: ResizeHandler<T>;
}
declare type ResizeApplication = ReturnType<typeof withResize>;
declare function withResize<E extends Event, T extends Constructor<BaseApplication<E>>>(Base: T): {
    new (...args: any[]): {
        canvas: HTMLCanvasElement;
        _events: Map<string, Map<string, ActionHandler<unknown>>>;
        _disposers: Set<() => void>;
        on(event: E): () => void;
        dispose(): void;
    };
    mapKey: string;
} & T;

interface TickHandler<T = TickApplication> extends ActionHandler<{
    app: T;
    dt: number;
}> {
}
interface TickEvent<T = TickApplication> extends Event {
    type: 'tick';
    action: TickHandler<T>;
}
declare type TickApplication = ReturnType<typeof withTick>;
declare function withTick<E extends Event, T extends Constructor<BaseApplication<E>>>(Base: T): {
    new (...args: any[]): {
        "__#1@#last": number;
        "__#1@#tickID": number | null;
        isRunning: boolean;
        "__#1@#tick": () => void;
        start(): void;
        stop(): void;
        canvas: HTMLCanvasElement;
        _events: Map<string, Map<string, ActionHandler<unknown>>>;
        _disposers: Set<() => void>;
        on(event: E): () => void;
        dispose(): void;
    };
    mapKey: string;
} & T;

declare type Point = {
    x: number;
    y: number;
};
interface InteractionHandler<T = InteractiveApplication> extends ActionHandler<{
    app: T;
    point: Point;
}> {
}
interface InteractionEvent<T = InteractiveApplication> extends Event {
    type: 'pointerdown' | 'pointerup' | 'pointermove';
    action: ({ app, point }: {
        app: T;
        point: Point;
    }) => void;
}
declare type InteractiveApplication = ReturnType<typeof withInteraction>;
declare function withInteraction<E extends Event, T extends Constructor<BaseApplication<E>>>(Base: T): {
    new (...args: any[]): {
        "__#2@#onPointerdown": (event: PointerEvent) => void;
        "__#2@#onPointerup": (event: PointerEvent) => void;
        "__#2@#onPointermove": (event: PointerEvent) => void;
        canvas: HTMLCanvasElement;
        _events: Map<string, Map<string, ActionHandler<unknown>>>;
        _disposers: Set<() => void>;
        on(event: E): () => void;
        dispose(): void;
    };
} & T;

export { ActionHandler, BaseApplication, CtxApplication, Event, InteractionEvent, InteractionHandler, InteractiveApplication, ResizeApplication, ResizeEvent, ResizeHandler, TickApplication, TickEvent, TickHandler, withInteraction, withResize, withTick };
